# Script to read a metadata record (text, XML, or SGML) and produce
# from it a web page containing the revision history that the
# metadata record describes.
#
# Peter N. Schweitzer (U.S. Geological Survey, Reston VA 20192)

# Revision history for this script:
#
# Ver  Date      Who  What
# ------------------------------------------------------------------------
# 1.5  20180927  PNS  Converted to use Bootstrap with new USGS Visual Identify
# 1.4  20121211  PNS  Added P3P and analytics information
# 1.3  20121019  PNS  Converted output to HTML5, modified styles
# 1.2  20060302  PNS  Added CSS stylesheets
# 1.1  20030723  PNS  Added summary attribute to main table for better
#                     accessibility.
#
# 1.0  20030321  PNS  Initial version shows date and description and
#                     ignores contact and sources used and produced.
#                     Note that banner and colors for page background
#                     and table background are hard-coded here, and
#                     URLs in the Process_Description texts are not
#                     converted into hypertext links.
# ------------------------------------------------------------------------

set version 1.5

package require mq

set stylesheets "  <link rel=\"stylesheet\" type=\"text/css\" href=\"/stylesheets/common.css\"/>\n  <link rel=\"stylesheet\" type=\"text/css\" href=\"/stylesheets/custom.css\"/>\n  <link rel=\"stylesheet\" type=\"text/css\" href=\"/bootstrap/css/bootstrap.min.css\"/>\n  <style type=\"text/css\"> table { border-collapse: collapse; } th, td { padding: 4px; } </style>\n  <!--\[if lt IE 9\]><script src=\"//html5shiv.googlecode.com/svn/trunk/html5.js\"></script><script src=\"https://oss.maxcdn.com/respond/1.4.2/respond.min.js\"></script><!\[endif\]-->\n  <script>(function(w,d,s,l,i){w\[l\]=w\[l\]||\[\];w\[l\].push({'gtm.start': new Date().getTime(),event:'gtm.js'});var f=d.getElementsByTagName(s)\[0\],j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src='https://www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);})(window,document,'script','dataLayer','GTM-TKQR8KP');</script>"
set header "<header id=\"navbar\" class=\"header-nav\"  role=\"banner\"><div class=\"tmp-container\"><div class=\"header-search\"><a class=\"logo-header\" href=\"https://www.usgs.gov/\" title=\"Home\"><img class=\"img\"  src=\"/images/logo.png\"  alt=\"Home\" /></a><form action=\"https://www.usgs.gov/science-explorer-results\" method=\"GET\" id=\"search-box\"><div class=\"fa-wrapper\"><label for=\"se_search\" class=\"only\">Search</label><input id=\"se_search\" type=\"search\" name=\"es\" placeholder=\"Search\"><button class=\"fa fa-search\" type=\"submit\"><span class=\"only\">Search</span></button></div></form></div></div></header>\n<div class=\"container\">"
set footer "</div> <!-- container -->\n\n<footer class=\"footer\"><div class=\"tmp-container\"><div class=\"footer-doi\"><ul class=\"menu nav\"><li class=\"first leaf menu-links menu-level-1\"><a href=\"https://www.doi.gov/privacy\">DOI Privacy Policy</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www.usgs.gov/laws/policies_notices.html\">Legal</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www2.usgs.gov/laws/accessibility.html\">Accessibility</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www.usgs.gov/sitemap.html\">Site Map</a></li><li class=\"last leaf menu-links menu-level-1\"><a href=\"https://answers.usgs.gov/\">Contact USGS</a></li></ul></div><hr/><div class=\"footer-doi\"><ul class=\"menu nav\"><li class=\"first leaf menu-links menu-level-1\"><a href=\"https://www.doi.gov/\">U.S. Department of the Interior</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www.doioig.gov/\">DOI Inspector General</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www.whitehouse.gov/\">White House</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www.whitehouse.gov/omb/e-gov/\">E-gov</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www.whitehouse.gov/open\">Open Government</a></li><li class=\"leaf menu-links menu-level-1\"><a href=\"https://www.doi.gov/pmb/eeo/no-fear-act\">No Fear Act</a></li><li class=\"last leaf menu-links menu-level-1\"><a href=\"https://www2.usgs.gov/foia\">FOIA</a></li></ul></div></div></footer>\n\n<noscript><iframe src=\"https://www.googletagmanager.com/ns.html?id=GTM-TKQR8KP\" height=\"0\" width=\"0\" style=\"display:none;visibility:hidden\"></iframe></noscript>"

if {$argc > 0} {
	set input_file [lindex $argv 0]
	} \
else {
	puts "Name the metadata file on the command line"
	exit
	}

proc show_date {ymd} {
	set month_name [list Error Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec]
	set y [string range $ymd 0 3]
	if {[string length $ymd] > 4} {
		if {[string index $ymd 4] == 0} {
			set m [string index $ymd 5]
			} \
		else {
			set m [string range $ymd 4 5]
			}
		set mon [lindex $month_name $m]
		if {[string length $ymd] > 6} {
			if {[string index $ymd 6] == 0} {
				set d [string index $ymd 7]
				} \
			else {
				set d [string range $ymd 6 7]
				}
			return "$d-$mon-$y"
			}
		return "$mon-$y"
		}
	return "$y"
	}

if {[metadata m -parse $input_file]} {
	if {[set p [$m find_first Identification_Information]]} {
		if {[set p [$m find_in $p Citation]]} {
			if {[set p [$m find_in $p Citation_Information]]} {
				if {[set p [$m find_in $p Title]]} {
					set title [$m value_of -nonewline $p]
					}
				}
			}
		}
	if {[set p [$m find_first Data_Quality_Information]]} {
		if {[set p [$m find_in $p Lineage]]} {
			if {[set q [$m find_in $p Process_Step]]} {
				set n 0
				while {$q} {
					if {[set r [$m find_in $q Process_Description]]} {
						set desc($n) [$m value_of $r]
						if {[set r [$m find_in $q Process_Date]]} {
							set date($n) [$m value_of -nonewline $r]
							if {$date($n) == ""} {
								set $date($n) "(no date given)"
								}
							} \
						else {
							set date($n) "(no date given)"
							}
						}
					incr n
					set q [$m find_next $q]
					}

				set nm1 [expr $n - 1]

				puts "<!DOCTYPE HTML>"
				puts "<html>"
				puts "  <meta charset=\"utf-8\"/>"
				puts "  <meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\"/>"
				puts "  <meta name=\"viewport\" content=\"width=device-width,initial-scale=1\"/>"
				puts "  <title>Revision history for [string map {> &gt; < &lt; & &amp;} $title]</title>"
				puts "$stylesheets"
				puts "</head>"
				puts "<body>"

				puts "$header"

				puts "<h1>[string map {> &gt; < &lt; & &amp;} $title]</h1>"
				puts "<h2>Revision history</h2>"

				puts "  <table class=\"table table-condensed\">"
				puts "    <thead>"

				puts "      <tr>"
				puts "        <th scope=\"col\">Date</th>"
				puts "        <th scope=\"col\">Description of revision</th>"
				puts "      </tr>"
				puts "    </thead>"
				puts "    <tbody>"

				for {set k $nm1} {$k >= 0} {incr k -1} {
					puts "      <tr style=\"vertical-align: top;\">"
					puts "        <th scope=\"row\" title=\"$date($k)\" style=\"white-space: nowrap;\">[show_date $date($k)]</th>"
					puts "        <td>"

					set lines [split $desc($k) "\n"]
					set line_count [llength $lines]
					set last_line [expr $line_count - 1]
					set preformatting 0
					for {set j 0} {$j < [llength $lines]} {incr j} {
						set line [lindex $lines $j]

						# At a blank line, print <p></p>, but if the next line begins with >, don't
						# Similarly, if the previous line began with >, don't

						if {$j < $last_line} {
							set next [lindex $lines [expr $j + 1]]
							} \
						else {
							set next ""
							}
						if {[string length $line] == 0} {
							if {$preformatting} {
								puts "</pre>"
								set preformatting 0
								} \
							else {
								if {[string index $next 0] != ">"} {
									puts "<br/>"
									}
								}
							} \
						else {
							if {[string index $line 0] == ">"} {
								if {!$preformatting} {
									puts "<pre>"
									set preformatting 1
									}
								set line [string range $line 1 end]
								} \
							else {
								if {$preformatting} {
									puts "</pre>"
									set preformatting 0
									}
								}
							set line [string map {> &gt; < &lt; & &amp;} $line]
							puts "$line"
							}
						}
					if {$preformatting} {
						puts "</pre>"
						}
					puts "        </td>"

					puts "      </tr>"
					}
				puts "    </tbody>"
				puts "  </table>"
				set generated_when [string trim [clock format [clock seconds] -format "%d-%b-%Y %H:%M"]]
				puts "<p>Generated by <a href=\"https://geo-nsdi.er.usgs.gov/metadata/whatsnew.tcl\">whatsnew.tcl</a> $version on $generated_when</p>"
				puts "$footer"
				puts "</body>"
				puts "</html>"
				}
			}
		}
	$m forget
	}

# end
